local Units={}
Relog.Units=Units

local MaxUpdateRate=0.1

local Roster = AceLibrary("Roster-2.1")

local Abbreviate={
	UnitID=true,
	lClass=true,
	Class=true,
	Faction=true,
	PowerType=true,
	RankName=true,
}

local NewEntry={}

function Units:AddUnitLog(Name)
	Relog.db.char.Units[Name].NumLogs=Relog.db.char.Units[Name].NumLogs+1
end

function Units:RemoveUnitLog(Name)
	if Relog.db.char.Units[Name] then
		Relog.db.char.Units[Name].NumLogs=Relog.db.char.Units[Name].NumLogs-1

		if Relog.db.char.Units[Name].NumLogs<=0 then
			Relog.db.char.Units[Name]=nil
		end
	end
end

function Units:UpdateUnitData(UnitID, Name, Time)
	local Unit=Relog.db.char.Units[Name]
	
	if Unit.LastUpdated>(Time-MaxUpdateRate) then
		return
	end

	Unit.LastUpdated=Time


	NewEntry.UnitID=UnitID


	if Roster:GetUnitObjectFromUnit(UnitID) then
		NewEntry.isGrouped=true
	else
		NewEntry.isGrouped=false
	end
	
	NewEntry.isPlayer=UnitIsPlayer(UnitID)
	NewEntry.Friendly=UnitIsFriend(UnitID,"player")
	NewEntry.lClass, NewEntry.Class=UnitClass(UnitID)
	_, NewEntry.Faction=UnitFactionGroup(UnitID)
	NewEntry.Level=UnitLevel(UnitID)
	NewEntry.Health=UnitHealth(UnitID)
	NewEntry.HealthMax=UnitHealthMax(UnitID)
	NewEntry.Power=UnitMana(UnitID)
	NewEntry.PowerMax=UnitManaMax(UnitID)
	NewEntry.PowerType=UnitPowerType(UnitID)
	NewEntry.RankNumber=UnitPVPRank(UnitID)
	NewEntry.RankName=GetPVPRankInfo(NewEntry.RankNumber,UnitID)

	for k,v in pairs(NewEntry) do
		if Abbreviate[k] then
			v=Relog.Abbrev:GetAbbrev(v)
		end

		if Unit[k][#Unit[k]]~=v then
			table.insert(Unit[k],v)
			table.insert(Unit[k.."Time"],Time)
		end
	end
end

function Units:UpdateUnit(UnitID, Time)
	if not UnitID or not UnitExists(UnitID) then
		return
	end

	local Name=UnitName(UnitID)
	
	Units:UpdateUnitData(UnitID, Name, Time)
end

function Units:UpdateUnitByName(Name, Time)
	local Unit=Relog.db.char.Units[Name]
	local UnitID

	--First Check Last Unit
	if type(Unit)~="table" then
		Unit={}
	end

	local LastUnitID
	if Unit.UnitID and #Unit.UnitID then
		LastUnitID=Relog.Abbrev:GetWord(Unit.UnitID[#Unit.UnitID])
	end

	if LastUnitID and UnitExists(LastUnitID) and UnitName(LastUnitID)==Name then
		UnitID=LastUnitID
	else
		--Next check our target
		if Name and Name==UnitName("target") then
			UnitID="target"
		else
			--Lastly check through our raids targets
			for unit in Roster:IterateRoster(true) do
				if UnitName(unit.unitid.."target")==Name then
					UnitID=unit.unitid.."target"
					break
				end
			end
		end
	end
	
	if not UnitID or not UnitExists(UnitID) then
		return
	end
	
	Units:UpdateUnitData(UnitID, Name, Time)
end

local Skip={
	NumLogs=true,
	LastUpdated=true,
	Last=true
}

local UnitCache={}
--Test /script Relog:PrintLiteral(Relog.Units:GetUnit("Ludith",1185057378,1))
function Units:GetUnit(Name, Time, CacheNum)
	local Unit

	if type(UnitCache[CacheNum])~="table" then
		UnitCache[CacheNum]={}
	end
	Unit=UnitCache[CacheNum]
	
	local UnitData=Relog.db.char.Units[Name]

	local VTime
	for k, v in pairs(UnitData) do
		if not Skip[k] and (not string.match(k,"Time$")) then
			Unit[k]=nil
			VTime=0

			local T, TimeData
			TimeData=UnitData[k.."Time"]
			
			for k2, Val in pairs(v) do
				T=TimeData[k2]
				if T and T<=Time and T>VTime then
					Unit[k]=Val
					VTime=T
				end				
			end

			if Unit[k] and Abbreviate[k] then
				Unit[k]=Relog.Abbrev:GetWord(Unit[k])
			end
		end
	end
	
	return Unit
end

function Units:DeleteBeforeTime(Time)
	for k, v in pairs(Relog.db.char.Units) do
		for k2, Section in pairs(v) do
			local TimePoint=nil			
			if not Skip[k] and (not string.match(k,"Time$")) then
				local T, TimeData
				TimeData=UnitData[k.."Time"]
	
				T=TimeData[1]

				while T<Time do
					table.remove(TimeData,1)
					table.remove(Section,1)
					T=TimeData[1]
				end
			end
		end
	end
end